import {
  GridEditable, Form, Select,
} from '@zeedhi/common';
import {
  Metadata, Loader, IDictionary, IEventParam,
} from '@zeedhi/core';

export class Pla05600PontoEntregaController {
  public utilities = Loader.getInstance('Utilities');

  public crudController = Loader.getInstance('CrudController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  get disabelNewRow() {
    const unidade = Metadata.getInstance<Select>('UNIDADE').value;
    if (!unidade) return true;
    return false;
  }
  
  public get setFilterUnidade() {
    return [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
  }
 
  public setFilterUnidadeDetail({ component }: IEventParam<Select>) {
    const values = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA'),
      NMFILIAL: localStorage.getItem('NMFILIALLOGADA'),
    };
    component.datasource.data = [values];
    component.setValue(localStorage.getItem('CDFILIALLOGADA'));
  }
  
  public reloadPqEntrega() {
    const gridPtEntrega: GridEditable = Metadata.getInstance('gridPontoEntrega');
    const unidade = Metadata.getInstance<Select>('UNIDADE').value;
    
    if (unidade !== null) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade,
        },
      ];
      
      gridPtEntrega.datasource.filter = conditions;
      gridPtEntrega.datasource.get();
    } else gridPtEntrega.datasource.data = [];
  }
  
  public salvaPtEntrega() {
    const gridPontoEntrega: GridEditable = Metadata.getInstance('gridPontoEntrega');
    const unidade = Metadata.getInstance<Select>('UNIDADE').value;
    
    const formPontoEntrega = Metadata.getInstance<Form>('formPontoEntrega');
    
    if (formPontoEntrega.validate()) {
      const editedRows: IDictionary = gridPontoEntrega.getEditedRows();
      editedRows.forEach((row: IDictionary) => {
        row.CDFILIAL = unidade;
        row.CDDESTFILI = row.CDDESTFILI.padStart('3', '0');
      });
      
      this.crudController.saveRows(editedRows, 'destfili', gridPontoEntrega).then(() => {
        this.gridEditController.defaultState(gridPontoEntrega);
        // gridPontoEntrega.datasource.get();
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
}
