import {
  Modal, ModalService, Button, Text, Radio, Number as NumberInput, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http, dayjs as moment,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00607CopiaPratosComensais {
  private modalCopiaPratoComensais!: Modal;
  
  public utilities = Loader.getInstance('Utilities');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  private gridEditableController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');
  
  private pratoSelecionado: IDictionary = {};
  
  private diasDoCardapio: IDictionary = {};
  
  public nomeTextPratoSelecionado = '';
  
  public visibilidadeComenPrato = true;
  
  private tabPratoOuComensaisFlag = true;
  
  public async createModal() {
    this.modalCopiaPratoComensais = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/pme00607_copia_pratos_comensais' });
  }
  
  public cancelModal() {
    // this.modalCopiaPratoComensais.hide();
    this.modalCopiaPratoComensais.destroy();
    this.modalCopiaPratoComensais = this.utilities.modalEmpty;
    this.elaboracaoCardapio.reloadCard();
  }
  
  public showModalCopiaPratoComensais({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    if (this.utilities.checkOpenModalOverlap()) this.validAndOpenModal();
  }
  
  private async validAndOpenModal() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    this.pratoSelecionado = cardapio.getCurrentSelectedRegister();
    
    this.diasDoCardapio = cardapio.dates.filter((data: IDictionary) => cardapio.dateLessDateOfProcess(data.DTCARDIAPME));
    this.tabPratoOuComensaisFlag = true;
    this.visibilidadeComenPrato = true;
      
    const idcard = cardapio.dates.filter((data: IDictionary) => data.DTCARDIAPME === this.pratoSelecionado.DTCARDIAPME);
    this.pratoSelecionado.filterData = this.elaboracaoCardapio.getFilterData;
    this.pratoSelecionado.QTCOMESTCPME = idcard[0].QTCOMESTCPME;
    
    if (this.pratoSelecionado.CDPRATO && cardapio.dateLessDateOfProcess(cardapio.dates.slice(-1)[0].DTCARDIAPME)) {
      if (this.elaboracaoCardapio.getFilterData.NRCARDPME !== undefined) {
        if (!this.modalCopiaPratoComensais) {
          this.createModal().then(() => {
            if (this.pratoSelecionado.NRCARDIAPME === null) this.buscaCardDia();
            this.modalCopiaPratoComensais.show();
          });
        } else {
          if (this.pratoSelecionado.NRCARDIAPME === null) this.buscaCardDia();
          this.modalCopiaPratoComensais.show();
        }
      } else {
        this.utilities.showAlert('withoutFilter', 'Operação bloqueada! Favor realizar o filtro.', 'error');
      }
    } else if (this.pratoSelecionado.CDPRATO) {
      this.utilities.showAlert('dateIsLessTheDateOfProcess', 'Data do cardapío fora do período de processamento.', 'error');
    } else {
      this.utilities.showAlert('pratoNaoInformado',
        'Operação bloqueada! Necessário posicionar em um prato planejado.',
        'error');
    }
  }
  
  public setaValorTextPrato({ component }: IEventParam<Text>) {
    component.text = this.pratoSelecionado.NMPRATO;
  }
  
  public setaValorDataPrato({ component }: IEventParam<Text>) {
    component.text = moment(
      this.pratoSelecionado.DTCARDIAPME.substring(0, 10), 'DD/MM/YYYY',
    ).format(this.utilities.getFormatLocaleFullDataWithWeek());
  }
  
  public setaValorComenPrato({ component }: IEventParam<NumberInput >) {
    component.value = parseInt(this.pratoSelecionado.valueInput, 10);
  }
  
  public verificaOpcaoRadioComensais({ component }: IEventParam<Radio>) {
    const radioValue = component.value;
    if (radioValue === '1') {
      this.visibilidadeComenPrato = false;
    } else {
      this.visibilidadeComenPrato = true;
    }
  }
  
  public alteraTab() {
    this.tabPratoOuComensaisFlag = !this.tabPratoOuComensaisFlag;
  }
  
  public confirmaAcaoCopiarPratos() {
    LoadingService.show(' ');

    if (this.tabPratoOuComensaisFlag) {
      this.realizaAcaoCopiaPratos();
    } else {
      this.realizaAcaoCopiaComensais();
    }
  }
  
  private realizaAcaoCopiaPratos() {
    const radioTabPratos = Metadata.getInstance<Radio>('radioComensais');
    const radioValue = radioTabPratos.value;
    const comensaisDoPrato = Metadata.getInstance<NumberInput>('textComensaisDoPrato').value;
    const sobreporValue = Metadata.getInstance<NumberInput>('checkboxSobrepor').value;
    const excecoes: IDictionary = this.buscaExcecoes();
    
    const rowParams = {
      CDTIPOCARD: this.pratoSelecionado.filterData.typeMenu.CDTIPOCARD,
      PRATOSELECIONADO: this.pratoSelecionado,
      COMENSAISPRATO: comensaisDoPrato,
      RADIOVALUE: radioValue,
      PERIODOCARD: this.diasDoCardapio,
      EXCECOES: excecoes,
      SOBREPOR: sobreporValue,
    };
    
    Http.post('/realizaAcaoCopiaPratos', { requestType: 'Row', row: rowParams }).then((response) => {
      if (response) {
        this.utilities.showAlert('plateSaved', 'Salvo com sucesso!', 'success');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      this.elaboracaoCardapio.reloadCard();
      this.cancelModal();
      LoadingService.hide();
    });
  }
  
  private realizaAcaoCopiaComensais() {
    const comensaisDoPrato = Metadata.getInstance<NumberInput>('inputComensaisDoprato').value;
    const excecoes: IDictionary = this.buscaExcecoes();
    
    const rowParams = {
      CDTIPOCARD: this.pratoSelecionado.filterData.typeMenu.CDTIPOCARD,
      PRATOSELECIONADO: this.pratoSelecionado,
      PERIODOCARD: this.diasDoCardapio,
      EXCECOES: excecoes,
      COMENSAISPRATO: comensaisDoPrato,
    };
    
    Http.post('/realizaAcaoCopiaComensais', { requestType: 'Row', row: rowParams }).then((response) => {
      if (response) {
        this.utilities.showAlert('plateSaved', 'Salvo com sucesso!', 'success');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      this.elaboracaoCardapio.reloadCard();
      this.cancelModal();
      LoadingService.hide();
    });
  }
  
  private buscaCardDia() {
    const row = this.pratoSelecionado;
    
    Http.post('/getCardia/get', { requestType: 'Row', row }).then((response) => {
      this.pratoSelecionado.NRCARDIAPME = response.data.dataset.cardiaOnClickPlate[0].NRCARDIAPME;
      this.pratoSelecionado.NRCARDPME = response.data.dataset.cardiaOnClickPlate[0].NRCARDPME;
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // pega as excecoes dos dias da semana
  private buscaExcecoes() {
    const flag = this.tabPratoOuComensaisFlag;
    const radioKeysArray = (flag === true)
      ? Metadata.getInstance<Radio>('checkboxExcecoes').value
      : Metadata.getInstance<Radio>('checkboxExcecoesTabComensais').value;
    const nomeDoElementoInput = (flag === true) ? 'textComensaisDoPrato' : 'textComensaisDoPratoTabComen';
    const excecoes: IDictionary = {};
    
    if (radioKeysArray.length > 1) {
      radioKeysArray.forEach((pos: string) => {
        if (pos !== '') {
          // preenche o objeto de excecoes
          excecoes[`${pos}`] = Metadata.getInstance<NumberInput>(`${nomeDoElementoInput}${pos}`).value;
        }
      });
    }
    
    return excecoes;
  }
}
